unit Functions;

interface

uses
  Messages, Windows, IniFiles, SysUtils, StdCtrls, Classes, ShlObj;

type
  PluginSettings = record
    GlobaldizFolder: string;
    GlobalDizFileName: string;
    UserColumnsFileNames: array [1 .. 10] of string;
    RefreshPanels: Boolean;
    ConfirmReplace: Boolean;
  end;

type
  internalSettings = record
    CLEAR: Boolean; // Clear main file
    GUI: Boolean; // show GUI
    USER: Boolean; // WORK WITH USER COLUMN (if false work with groups)
    USERFILE: string; // User column filename
    GROUP: string; // group name
    DIZ: string; // file description
    FILENAME: string; // filename
    AUTODETECTFILENAME: Boolean;
    // if "/file" parameter passed, then disable filename autodetect
    LIST: Boolean; // if true, then listfile passed
    DELETE: Boolean; // delete file from group
    OPTIONS: Boolean; // show settings dialog
    HELP: Boolean; // Show Help
    HELPONERROR: string; // error in command-line;
  end;

function GetSettingsFile: string;
procedure GetSettings(var Settings: PluginSettings);
procedure MCreateFile(FILENAME: string);
procedure GetGroups(var Groups: TCombobox; Settings: PluginSettings);
function IsTCListFile(FILENAME: string): Boolean;
procedure ClearDiz(Settings: PluginSettings);
procedure DelDiz(FromFile, ForFile: string; var iset: internalSettings);
procedure SaveUserDiz(DizFile, ForFile, newdiz: string;
  Settings: PluginSettings; iset: internalSettings);
procedure SaveDiz(GROUP, ForFile, newdiz: string; Settings: PluginSettings;
  iset: internalSettings);
procedure GetUserColumns(var UserColumns: TCombobox; Settings: PluginSettings);
function AddContextMenu(ItemName, RunString: string): string;
function GiveFolder(caption: string; parent: HWND): string;
procedure Reread_need;

var
  Settings: PluginSettings;
  Usergroup: Boolean = False;
  iset: internalSettings;
  FirstList: Boolean = True;
  Codepage: Integer = 0;

implementation

function GetSettingsFile: string;
var
  dwKeySize: DWORD;
  Key: HKEY;
  dwType: DWORD;
begin
  //Result := ChangeFileExt(GetModuleName(HInstance), '.ini');
  Result := ExtractFilePath(GetModuleName(HInstance) + 'global_diz.ini');
  {
  if RegOpenKeyEx(hKey_Current_User, 'Software\Legion\GlobalDiz', 0, KEY_READ,
    Key) = ERROR_SUCCESS then
    try
      RegQueryValueEx(Key, 'IniFileName', nil, @dwType, nil, @dwKeySize);
      if (dwType in [REG_SZ, REG_EXPAND_SZ]) and (dwKeySize > 1) then
      begin
        SetLength(Result, dwKeySize - 1);
        RegQueryValueEx(Key, 'IniFileName', nil, @dwType, PByte(PChar(Result)),
          @dwKeySize);
        Result := Result;
      end;
    finally
      RegCloseKey(Key);
    end;
  }
end;

{
procedure MCreateFile(FILENAME: string);
var
  x: TextFile;
begin
  try
    AssignFile(x, FILENAME);
    Rewrite(x);
    CloseFile(x);
  except
    MessageBox(0, PChar('Can`t create ' + FILENAME + '.'), 'Error',
      mb_ok + MB_ICONINFORMATION);
    Exit;
  end;
end;
}

procedure MCreateFile(FileName: string);
const
  PreambleUTF8: array [0..6] of Byte = ($EF, $BB, $BF, $3B, $2A, 13, 10);
  PreambleLE: array [0..9] of Byte = ($FF, $FE, $3B, 0, $2A, 0, 13, 0, 10, 0);
var
  x: TFileStream;
begin
  try
    x := TFileStream.Create(FileName, fmCreate);
    try
      if Codepage = CP_UTF8 then
        x.Write(PreambleUTF8[0], SizeOf(PreambleUTF8))
      else if Codepage = 1200 then
        x.Write(PreambleLE[0], SizeOf(PreambleLE));
    finally
      x.Free;
    end;
  except
    // do nothing
  end;
end;

procedure GetSettings(var Settings: PluginSettings);
var
  Ini: Tinifile;
  MaxUserColumns: Integer;
  t1: string;
begin
  t1 := IncludeTrailingBackslash(ExtractFilePath(ParamStr(0))) +
    'global_diz.ini';
  if not FileExists(t1) then
    Ini := Tinifile.Create(GetSettingsFile)
  else
    Ini := Tinifile.Create(t1);
  Codepage := INI.ReadInteger('main', 'CodePage', CodePage);
  if Codepage <> 0 then
    if CodePage <> 1200 then
      CodePage := 0;
  Settings.GlobaldizFolder := IncludeTrailingBackslash
    (Ini.ReadString('main', 'globaldizfolder', ''));
  if not DirectoryExists(Settings.GlobaldizFolder) then
    Settings.GlobaldizFolder := '\';
  if Settings.GlobaldizFolder = '\' then
    Settings.GlobaldizFolder := IncludeTrailingBackslash
      (ExtractFilePath(ParamStr(0)));
  Settings.GlobalDizFileName := Settings.GlobaldizFolder + 'global.diz';
  if not FileExists(Settings.GlobalDizFileName) then
    MCreateFile(Settings.GlobalDizFileName);
  Settings.RefreshPanels := Ini.ReadBool('main', 'RefreshPanels', False);
  Settings.ConfirmReplace := Ini.ReadBool('main', 'ConfirmReplace', False);
  for MaxUserColumns := 1 to 10 do
  begin
    if Ini.ReadString('UserColumns', 'Column' + inttostr(MaxUserColumns), '') <> ''
    then
      Settings.UserColumnsFileNames[MaxUserColumns] := Settings.GlobaldizFolder
        + Ini.ReadString('UserColumns',
        'Column' + inttostr(MaxUserColumns), '');
  end;
  Ini.Destroy;
end;

procedure GetGroups(var Groups: TCombobox; Settings: PluginSettings);
var
  DIZ: TextFile;
  s: string;
begin
  Groups.CLEAR;
  if not FileExists(Settings.GlobalDizFileName) then
    MCreateFile(Settings.GlobalDizFileName);
  AssignFile(DIZ, Settings.GlobalDizFileName, CodePage);
  Reset(DIZ);
  while not eof(DIZ) do
  begin
    Readln(DIZ, s);
    if (length(s) < 1) or (s[1] = #13) then
      continue;
    if (s[1] = '[') and (s[length(s)] = ']') then
    begin
      DELETE(s, 1, 1);
      DELETE(s, length(s), 1);
      Groups.Items.Add(s);
    end;
  end;
  CloseFile(DIZ);
end;

procedure GetUserColumns(var UserColumns: TCombobox; Settings: PluginSettings);
var
  i: byte;
begin
  UserColumns.CLEAR;
  for i := 1 to 10 do
  begin
    if Settings.UserColumnsFileNames[i] <> '' then
      UserColumns.Items.Add
        (Copy(ExtractFileName(Settings.UserColumnsFileNames[i]), 1,
        pos('.', ExtractFileName(Settings.UserColumnsFileNames[i])) - 1))
    else
      Exit;
  end;
end;

function IsTCListFile(FILENAME: string): Boolean;
begin
  Result := False;
  if (Copy(LowerCase(ExtractFileName(FILENAME)), 1, 3) = 'cmd') and
    (ExtractFileExt(FILENAME) = '.tmp') then
    Result := True;
end;

function GetGroupForFile(FILENAME: shortstring;
  Settings: PluginSettings): string;
var
  s, s1: string;
  DIZ: TextFile;
begin
  if not FileExists(Settings.GlobalDizFileName) then
    MCreateFile(Settings.GlobalDizFileName);
  AssignFile(DIZ, Settings.GlobalDizFileName, CodePage);
  Result := '';
  Reset(DIZ);
  s1 := '';
  while not eof(DIZ) do
  begin
    try
      Readln(DIZ, s);
    except
      if s[1] = #13 then
        continue;
    end;
    if s = '' then
      continue;
    if (s[1] = '[') then
      s1 := s;
    // if lowercase(s)='[none]' then s1:='';
    if LowerCase(ExtractShortPathName(Copy(s, 1, pos('=', s) - 1)))
      = LowerCase(ExtractShortPathName(FILENAME)) then
    begin
      Result := Copy(s1, 2, length(s1) - 2);
      CloseFile(DIZ);
      Exit;
    end;
  end;
  CloseFile(DIZ);
end;

procedure SaveDiz(GROUP, ForFile, newdiz: string; Settings: PluginSettings;
  iset: internalSettings);
var
  AskChange, GroupChange: Boolean;
  DIZ: Tinifile;
  tmp: TextFile;
  s, t: string;
begin
  if not FileExists(Settings.GlobalDizFileName) then
    MCreateFile(Settings.GlobalDizFileName);
  if GROUP = '' then
  begin
    GROUP := GetGroupForFile(ForFile, Settings);
    if GROUP = '' then
      GROUP := 'NONE';
  end;
  DIZ := Tinifile.Create(Settings.GlobalDizFileName);

  if (iset.LIST) or IsTCListFile(ForFile) then
  begin
    GroupChange := False;
    AskChange := True;
    AssignFile(tmp, ForFile, CodePage);
    Reset(tmp);
    while not eof(tmp) do
    begin
      Readln(tmp, s);
      if s[length(s)] = '\' then
        DELETE(s, length(s), 1);
      t := GetGroupForFile(s, Settings);
      // if t=group then continue;
      if (t <> '') and (not GroupChange) then
      begin
        if not Settings.ConfirmReplace then
          GroupChange := True;
        if (Settings.ConfirmReplace) and (AskChange) then
        begin
          AskChange := False;
          case MessageBox(0, PChar('File ' + s + ' from listfile ' + ForFile +
            ' already exists in ' + t +
            ' group. Do you want change group for all files to ' + GROUP +
            ' group?'), PChar('Change group ' + t + ' to ' + GROUP),
            MB_YESNO + MB_ICONQUESTION) of
            ID_YES:
              GroupChange := True;
            ID_NO:
              continue;
          end;
        end;
      end;
      if (GroupChange) then
        DIZ.DeleteKey(t, s)
      else if (t <> '') and (Settings.ConfirmReplace) then
        continue;
      DIZ.WriteString(GROUP, s, newdiz);
    end;
    CloseFile(tmp);
    DIZ.Destroy;
    Exit;
  end
  else
  begin
    s := GetGroupForFile(ForFile, Settings);
    // if s=group then exit;
    if s <> '' then
    begin
      if Settings.ConfirmReplace then
      begin
        case MessageBox(0, PChar('File ' + ForFile + ' already exists in ' + s +
          ' group. Do you want change group to ' + GROUP + '?'),
          PChar('Change group ' + t + ' to ' + GROUP),
          MB_YESNO + MB_ICONQUESTION) of
          // ID_YES:diz.DeleteKey(s,forfile);
          ID_NO:
            Exit;
        end;
      end;
      DIZ.DeleteKey(s, ForFile);
      DIZ.DeleteKey(s, ExtractShortPathName(ForFile)); //   
    end;
    DIZ.WriteString(GROUP, ForFile, newdiz);
    DIZ.Destroy;
  end;
end;

procedure DelDiz(FromFile, ForFile: string; var iset: internalSettings);
var
  DIZ, newdiz, tmp: TextFile;
  t, s: string;
  temp1, temp2: string;
begin
  if not FileExists(FromFile) then
    MCreateFile(FromFile);

  if ((iset.LIST) or IsTCListFile(ForFile)) and (FirstList) then
  begin
    AssignFile(tmp, ForFile, Codepage);
    Reset(tmp);
    while not eof(tmp) do
    begin
      FirstList := False;
      Readln(tmp, s);
      if s[length(s)] = '\' then
        DELETE(s, length(s), 1);
      DelDiz(FromFile, s, iset);
    end;
    CloseFile(tmp);
  end
  else
  begin

    AssignFile(DIZ, FromFile, Codepage);
    AssignFile(newdiz, FromFile + '~', Codepage);
    Rewrite(newdiz);
    Reset(DIZ);
    while not eof(DIZ) do
    begin
      Readln(DIZ, s);
      t := Copy(s, 1, pos('=', s) - 1);
      temp1 := LowerCase(ExtractShortPathName(ForFile));
      temp2 := LowerCase(ExtractShortPathName(t));
      if temp1 = temp2 then
      begin

      end
      else
      begin
        writeln(newdiz, s);
      end;
    end;
    CloseFile(DIZ);
    CloseFile(newdiz);
    deletefile(FromFile);
    renamefile(FromFile + '~', FromFile);
  end;
end;

procedure SaveUserDiz(DizFile, ForFile, newdiz: string;
  Settings: PluginSettings; iset: internalSettings);
var
  Ini: TIniFile;
  DIZ: TextFile;
  x: byte;
  i: Integer;
  st: TStringList;
  s: string;
  function UserColumnExists(ColumnFileName: string;
    var FreeColumn: byte): Boolean;
  var
    i: byte;
  begin
    Result := False;
    for i := 1 to 10 do
    begin
      if Settings.UserColumnsFileNames[i] = '' then
      begin
        FreeColumn := i;
        Result := False;
        Exit;
      end;
      if Settings.UserColumnsFileNames[i] = ColumnFileName then
      begin
        Result := True;
        Exit;
      end;
    end;
  end;

begin
  if not FileExists(DizFile) then
    MCreateFile(DizFile);
  DelDiz(DizFile, ForFile, iset);
  if (iset.LIST) or IsTCListFile(ForFile) then
  begin
    AssignFile(DIZ, DizFile, Codepage);
    append(DIZ);
    st := TStringList.Create;
    st.LoadFromFile(ForFile);
    for i := 0 to st.Count - 1 do
    begin
      s := st[i];
      if s[length(s)] = '\' then
        DELETE(s, length(s), 1);
      writeln(DIZ, s + '=' + newdiz);
    end;
    st.Free;
    CloseFile(DIZ);
    Exit;
  end;
  AssignFile(DIZ, DizFile, Codepage);
  append(DIZ);
  writeln(DIZ, ForFile + '=' + newdiz);
  CloseFile(DIZ);
  if (not UserColumnExists(DizFile, x)) and (x < 11) then
  begin
    Ini := Tinifile.Create(GetSettingsFile);
    Ini.WriteString('UserColumns', 'Column' + inttostr(x),
      ExtractFileName(DizFile));
    Ini.Free;
  end;
end;

procedure ClearDiz(Settings: PluginSettings);
var
  DIZ, newdiz: TextFile;
  s, t: string;
begin
  if not FileExists(Settings.GlobalDizFileName) then
    MCreateFile(Settings.GlobalDizFileName);
  AssignFile(DIZ, Settings.GlobalDizFileName, CodePage);
  AssignFile(newdiz, Settings.GlobalDizFileName + '~', CodePage);
  Rewrite(newdiz);
  Reset(DIZ);
  while not eof(DIZ) do
  begin
    Readln(DIZ, s);
    t := Copy(s, 1, pos('=', s) - 1);
    if (not(FileExists(t) or DirectoryExists(t))) and (t <> '') then
    begin
      { writeln (diz);
        messagebox (application.Handle,pchar (t),'File dont exists',0); }
      // do nothing
    end
    else
      writeln(newdiz, s);
  end;
  CloseFile(DIZ);
  CloseFile(newdiz);
  deletefile(Settings.GlobalDizFileName);
  renamefile(Settings.GlobalDizFileName + '~', Settings.GlobalDizFileName);
end;

function AddContextMenu(ItemName, RunString: string): string;
var
  Key: HKEY;
begin
  if RegCreateKeyEx(hKey_Classes_Root, '*\shell\ADD_DIZ\', 0, nil,
    REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, nil, Key, nil) = ERROR_SUCCESS then
    try
      RegSetValueEx(Key, '', 0, REG_SZ, @ItemName[1], length(ItemName));
    finally
      RegCloseKey(Key);
    end;
  if RegCreateKeyEx(hKey_Classes_Root, '*\shell\ADD_DIZ\command\', 0, nil,
    REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, nil, Key, nil) = ERROR_SUCCESS then
    try
      RegSetValueEx(Key, '', 0, REG_SZ, @RunString[1], length(RunString));
    finally
      RegCloseKey(Key);
    end;
  if RegCreateKeyEx(hKey_Classes_Root, 'Folder\shell\ADD_DIZ\', 0, nil,
    REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, nil, Key, nil) = ERROR_SUCCESS then
    try
      RegSetValueEx(Key, '', 0, REG_SZ, @ItemName[1], length(ItemName));
    finally
      RegCloseKey(Key);
    end;
  if RegCreateKeyEx(hKey_Classes_Root, 'Folder\shell\ADD_DIZ\command\', 0, nil,
    REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, nil, Key, nil) = ERROR_SUCCESS then
    try
      RegSetValueEx(Key, '', 0, REG_SZ, @RunString[1], length(RunString));
    finally
      RegCloseKey(Key);
    end;
end;

function GiveFolder(caption: string; parent: HWND): string;
var
  lpItemID: PItemIDList;
  BrowseInfo: TBrowseInfo;
  DisplayName: array [0 .. MAX_PATH] of char;
  TempPath: array [0 .. MAX_PATH] of char;
begin
  Result := '';
  DisplayName := 'TWinAmp';
  FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);
  BrowseInfo.hwndOwner := parent;
  BrowseInfo.pszDisplayName := @DisplayName;
  BrowseInfo.lpszTitle := PChar(caption);
  BrowseInfo.ulFlags := BIF_RETURNONLYFSDIRS;
  lpItemID := SHBrowseForFolder(BrowseInfo);
  if lpItemID <> nil then
  begin
    SHGetPathFromIDList(lpItemID, TempPath);
    Result := TempPath;
    GlobalFreePtr(lpItemID);
  end;
end;

procedure Reread_need;
  function FindTCWindow: HWND;
  begin
    Result := FindWindow('TTOTAL_CMD', nil);
  end;
  procedure ReReadPanels;
  begin
    PostMessage(FindTCWindow, WM_USER + 51, 540, 0);
  end;

begin
  if Settings.RefreshPanels then
    ReReadPanels;
end;

begin

end.
