library global_diz;


{$E wdx}
{$IFDEF WIN64}
  {$E w64}
  {$IFNDEF VER230}
    {$E wdx64}
  {$ENDIF}
{$ENDIF}

{$R *.res}

uses
  SysUtils,
  IniFiles,
  Windows,
  Classes,
  contplug;

type
  PluginSettings = record
    GlobalDizFolder: string;
    GlobaldizFileName: string;
    UserColumnsFileNames: array [1 .. 10] of string;
  end;

const
  BufferSize = 8192;

var
  Settings: PluginSettings;
  CodePage: Word = 0;
  DelayedLoad: Boolean = False;
  RTL: _RTL_CRITICAL_SECTION;

{
procedure MCreateFile(FileName: string);
var
  x: File;
begin
  AssignFile(x, FileName);
  Rewrite(x);
  CloseFile(x);
end;
}

procedure MCreateFile(const FileName: string);
const
  PreambleUTF8: array [0..6] of Byte = ($EF, $BB, $BF, $3B, $2A, 13, 10);
  PreambleLE: array [0..9] of Byte = ($FF, $FE, $3B, 0, $2A, 0, 13, 0, 10, 0);
var
  x: TFileStream;
begin
  try
    x := TFileStream.Create(FileName, fmCreate);
    try
      if Codepage = CP_UTF8 then
        x.Write(PreambleUTF8[0], SizeOf(PreambleUTF8))
      else if Codepage = 1200 then
        x.Write(PreambleLE[0], SizeOf(PreambleLE));
    finally
      x.Free;
    end;
  except
    // do nothing
  end;
end;

procedure GetSettings;
var
  Ini: TIniFile;
  MaxUserColumns: Integer;
  t1: string;
  Inifilename: string;
begin
  Inifilename := GetModuleName(HInstance);
  t1 := IncludeTrailingPathDelimiter(ExtractFilePath(Inifilename)) +
    'global_diz.ini';
  Ini := TIniFile.Create(t1);
  try
    Codepage := Ini.ReadInteger('main', 'CodePage', CodePage);
    DelayedLoad := Ini.ReadBool('main', 'DelayedLoad', DelayedLoad);
    if Codepage <> 0 then
      if CodePage <> 1200 then
        CodePage := 0;
    Settings.GlobalDizFolder := IncludeTrailingPathDelimiter
      (Ini.ReadString('main', 'globaldizfolder', ''));
    if not DirectoryExists(Settings.GlobalDizFolder) then
      Settings.GlobalDizFolder := '\';
    if Settings.GlobalDizFolder = '\' then
      Settings.GlobalDizFolder := IncludeTrailingPathDelimiter
        (ExtractFilePath(Inifilename));
    Settings.GlobaldizFileName := Settings.GlobalDizFolder + 'global.diz';
    if not FileExists(Settings.GlobaldizFileName) then
      MCreateFile(Settings.GlobaldizFileName);
    for MaxUserColumns := 1 to 10 do
    begin
      if Ini.ReadString('UserColumns', 'Column' + IntToStr(MaxUserColumns), '')
        <> '' then
        Settings.UserColumnsFileNames[MaxUserColumns] :=
          Settings.GlobalDizFolder + Ini.ReadString('UserColumns',
          'Column' + IntToStr(MaxUserColumns), '');
    end;
  finally
    Ini.Free;
  end;
end;

function ContentGetSupportedField(FieldIndex: Integer; FieldName: PAnsiChar;
  Units: PAnsiChar; maxlen: Integer): Integer; stdcall;
begin
  case FieldIndex of
    0:
      begin
        StrLCopy(FieldName, 'Global description', maxlen - 1);
        Result := ft_stringW;
      end;
    1:
      begin
        StrLCopy(FieldName, 'File group', maxlen - 1);
        Result := ft_stringW;
      end;
  else
    if Settings.UserColumnsFileNames[FieldIndex - 1] = '' then
      Result := ft_nomorefields
    else
    begin
      StrLCopy(FieldName,
        PAnsiChar(AnsiString(Copy(ExtractFileName(Settings.UserColumnsFileNames
        [FieldIndex - 1]), 1, Pos('.',
        ExtractFileName(Settings.UserColumnsFileNames[FieldIndex - 1])) - 1))),
        maxlen - 1);
      Result := ft_stringW;
    end;
  end;
  StrCopy(Units, '');
end;

function GetShortPathName(const FileName: string): string; inline;
begin
  //Result := ExtractShortPathName(FileName);
  Result := AnsiLowerCase(FileName);
end;

function GetGlobalDizForFile(FileName: string): string; inline;
var
  diz: TextFile;
  s, t, shortfn: string;
  Buf: array [0 .. BufferSize - 1] of Byte;
begin
  Result := '';
  try
    shortfn := GetShortPathName(FileName);

    if not FileExists(Settings.GlobaldizFileName) then
      MCreateFile(Settings.GlobaldizFileName);
    AssignFile(diz, Settings.GlobaldizFileName, CodePage);
    Reset(diz);
    try
      SetTextBuf(diz, Buf, SizeOf(Buf));
      while not Eof(diz) do
      begin
        Readln(diz, s);
        t := Copy(s, 1, Pos('=', s) - 1);
        if SameText(GetShortPathName(t), shortfn) then
        // if t = FileName then
        begin
          Result := Copy(s, Pos('=', s) + 1, Length(s));
          Exit;
        end;
      end;
    finally
      CloseFile(diz);
    end;
  except
    // do nothing
  end;
end;

function GetGroupForFile(FileName: string): string; inline;
var
  s, s1, shortfn: string;
  diz: TextFile;
  Buf: array [0 .. BufferSize - 1] of Byte;
begin
  Result := '';
  try
    shortfn := GetShortPathName(FileName);
    if not FileExists(Settings.GlobaldizFileName) then
      MCreateFile(Settings.GlobaldizFileName);
    AssignFile(diz, Settings.GlobaldizFileName, CodePage);
    Reset(diz);
    try
      SetTextBuf(diz, Buf, SizeOf(Buf));
      s1 := '';
      while not Eof(diz) do
      begin
        try
          Readln(diz, s);
        except
          if s[1] = #13 then
            Continue;
        end;
        if s = '' then
          Continue;
        if (s[1] = '[') then
          s1 := s;
        if LowerCase(s) = '[none]' then
          s1 := '';
        s := Copy(s, 1, Pos('=', s) - 1);
        if SameText(GetShortPathName(s), shortfn) then
        // if s = FileName then
        begin
          Result := Copy(s1, 2, Length(s1) - 2);
          Exit;
        end;
      end;
    finally
      CloseFile(diz);
    end;
  except
    // do nothing
  end;
end;

function GetUserDizForFile(FileName: string; columnnum: Byte): string; inline;
var
  diz: TextFile;
  s, t, shortfn: string;
  Buf: array [0 .. BufferSize - 1] of Byte;
begin
  Result := '';
  try
    shortfn := GetShortPathName(FileName);

    if not FileExists(Settings.UserColumnsFileNames[columnnum]) then
      MCreateFile(Settings.UserColumnsFileNames[columnnum]);
    AssignFile(diz, Settings.UserColumnsFileNames[columnnum], CodePage);
    Reset(diz);
    try
      SetTextBuf(diz, Buf, SizeOf(Buf));
      while not Eof(diz) do
      begin
        Readln(diz, s);
        t := Copy(s, 1, Pos('=', s) - 1);
        if SameText(GetShortPathName(t), shortfn) then
        // if t = FileName then
        begin
          Result := Copy(s, Pos('=', s) + 1, Length(s));
          Exit;
        end;
      end;
    finally
      CloseFile(diz);
    end;
  except
    // do nothing
  end;
end;

function ContentGetValueW(FileName: PWideChar; FieldIndex, UnitIndex: Integer;
  FieldValue: PByte; maxlen, flags: Integer): Integer; stdcall;
begin
  Result := ft_fieldempty;

  if DelayedLoad then
  begin

    if (flags and CONTENT_DELAYIFSLOW > 0) then
    begin
      Result := ft_delayed;
      Exit;
    end;

    EnterCriticalSection(RTL);
  end;

  try

    case FieldIndex of
      0:
        begin
          StrLCopy(PChar(FieldValue),
            PChar(GetGlobalDizForFile(StrPas(FileName))), maxlen div 2 - 2);
          Result := ft_stringW;
        end;
      1:
        begin
          StrLCopy(PChar(FieldValue), PChar(GetGroupForFile(FileName)),
            maxlen div 2 - 2);
          Result := ft_stringW;
        end;
    else
      begin
        StrLCopy(PChar(FieldValue), PChar(GetUserDizForFile(FileName,
          FieldIndex - 1)), maxlen div 2 - 2);
        Result := ft_stringW;
      end;
    end;

  finally
    if DelayedLoad then
      LeaveCriticalSection(RTL);
  end;
end;

function ContentGetValue(FileName: PAnsiChar; FieldIndex, UnitIndex: Integer;
  FieldValue: PByte; maxlen, flags: Integer): Integer; stdcall;
begin
  Result := ContentGetValueW(PWideChar(WideString(AnsiString(FileName))),
    FieldIndex, UnitIndex, FieldValue, maxlen, flags);
end;

procedure DllInit(Code: Integer);
begin
  case Code of
    DLL_PROCESS_ATTACH:
    begin
      try
        GetSettings;
      except
      end;
      if DelayedLoad then
        InitializeCriticalSection(RTL);
    end;
    DLL_PROCESS_DETACH:
    begin
      if DelayedLoad then
        DeleteCriticalSection(RTL);
    end;
  end; // case
end;

exports
  // ContentGetDetectString,
  ContentGetSupportedField,
  ContentGetValue,
  ContentGetValueW;

{ ContentSetDefaultParams,
  ContentGetDefaultSortOrder; }

begin
  //ReportMemoryLeaksOnShutdown := True;
  DllProc := @DllInit;
  DllInit(DLL_PROCESS_ATTACH);
end.

